/* verilator lint_off WIDTH */
/* verilator lint_off UNOPTFLAT */

module fpualu
(
    input    [15:0]  a, // Операнд A
    input    [15:0]  b, // Операнд B
    output   [15:0]  c  // Результат C
);

// -----------------------------------------------------------------------------
wire        sa = a[   15], sb = b[   15]; // Знак +/-
wire [4:0]  ea = a[14:10], eb = b[14:10]; // Порядок 5 бит
wire [9:0]  ma = a[  9:0], mb = b[  9:0]; // Значение мантиссы 10 бит
wire        da = (ea == 1'b0),
            db = (eb == 1'b0);

// -----------------------------------------------------------------------------
wire [21:0] m0 = {!da, ma} * {!db, mb};
wire [ 5:0] e0 = ea + eb;
wire        sn = sa ^ sb;
// -----------------------------------------------------------------------------

// Нормализация полученного числа
wire [31:0] m1 = {m0[21] ? m0[21:1] : m0[20:0], 11'b0000000000};

// Подсчет количества лидирующих нулей в m1
wire [4:0]  np = {
    ~|m1[31:16], // 4
    ~|m2[31:24], // 3
    ~|m3[31:28], // 2
    ~|m4[31:30], // 1
     ~m5[31]     // 0
};

wire [31:0] m2 = np[4] ? {m1[15:0], 16'b0} : m1;
wire [31:0] m3 = np[3] ? {m2[23:0],  8'b0} : m2;
wire [31:0] m4 = np[2] ? {m3[27:0],  4'b0} : m3;
wire [31:0] m5 = np[1] ? {m4[29:0],  2'b0} : m4;
wire [31:0] m6 = np[0] ? {m5[30:0],  1'b0} : m5;

// Итоговый порядок
wire [ 6:0] e1 = e0 + (da + db) + m0[21] - 15 - np;

// Коррекция денормализованного числа
wire        dn = e1[6] || e1 == 0;
wire [ 5:0] e2 = 1 - e1;
wire [ 9:0] rm = dn ? m6[31:21] >> e2 : m6[30:21];
wire        vp = e1[6:5] == 2'b01 || e1 == 31 || &ea || &eb;

// Результат
assign c = vp ? {sn, 5'h1F, 10'b0} : {sn, dn ? 5'b00000 : e1[4:0], rm};

endmodule

